module GrapePathHelpers
  # methods to extend Grape::Endpoint so that calls
  # to unknown methods will look for a route with a matching
  # helper function name
  module NamedRouteMatcher
    def method_missing(method_id, *args)
      return super unless method_id.to_s =~ /_path$/

      segments = args.first || {}

      return super unless segments.is_a?(Hash)

      requested_segments = segments.keys.map(&:to_s)

      route = Grape::API::Instance.decorated_routes.detect do |r|
        route_match?(r, method_id, requested_segments)
      end

      if route
        route.send(method_id, *args)
      else
        super
      end
    end
    ruby2_keywords(:method_missing)

    def respond_to_missing?(method_name, _include_private = false)
      return super unless method_name =~ /_path$/

      Grape::API::Instance.decorated_routes.detect do |route|
        return true if route.respond_to?(method_name)
      end

      super
    end

    def route_match?(route, method_name, requested_segments)
      return false unless route.respond_to?(method_name)

      route.uses_segments_in_path_helper?(requested_segments)
    end
  end
end
